@@ -1,6 +1,6 @@
 /*
  *  Methods for LUKS-related key-management for cryptmount
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -42,6 +42,9 @@
 #  include "cmtesting.h"
 #endif
 
+/*! \addtogroup keymgrs
+ *  @{ */
+
 
 typedef struct {
     unsigned keyslot;
@@ -228,9 +231,9 @@ static int kmluks_get_key(bound_tgtdefn_t *boundtgt,
 
     crypt_init(&luks_ctxt, tgtdev);
     lukshdr = crypt_get_lukshdr(luks_ctxt);
-    LUKS_read_phdr(tgtdev, lukshdr, 1, luks_ctxt);
-    slot = LUKS_open_key_with_hdr(tgtdev, -1, passwd, strlen(passwd),
-                    lukshdr, &lukskey, luks_ctxt);
+    LUKS_read_phdr(lukshdr, 1, 0, luks_ctxt);
+    slot = LUKS_open_key_with_hdr(-1, passwd, strlen(passwd),
+                                lukshdr, &lukskey, luks_ctxt);
     if (slot < 0) {
         fprintf(stderr, _("Failed to extract LUKS key for \"%s\" (errno=%d)\n"),
                 tgt->ident, -slot);
@@ -324,13 +327,13 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
         unsigned long reqd_align = DEFAULT_DISK_ALIGNMENT, align_offs = 0;
 
         keyslot = 0;
-        get_topology_alignment(tgtdev, &reqd_align,
+        device_topology_alignment(crypt_get_device(luks_ctxt), &reqd_align,
                                 &align_offs, DEFAULT_DISK_ALIGNMENT);
         kmluks_splitmode(boundtgt->tgt->cipher, &ciphername, &ciphermode);
         r = LUKS_generate_phdr(lukshdr, lukskey, ciphername, ciphermode,
                             boundtgt->tgt->key.digestalg, NULL, LUKS_STRIPES,
                             reqd_align / SECTOR_SIZE, align_offs / SECTOR_SIZE,
-                            *iteration_time, PBKDF2_per_sec, NULL, luks_ctxt);
+                            *iteration_time, PBKDF2_per_sec, 0, luks_ctxt);
         if (r < 0) {
             fprintf(stderr, _("Failed to create LUKS header for \"%s\"\n"),
                     boundtgt->tgt->ident);
@@ -341,7 +344,7 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
         /* Changing password on existing partition */
         keyslot = (luksor != NULL ? luksor->keyslot : 0);
 
-        if (LUKS_read_phdr(tgtdev, lukshdr, 1, luks_ctxt) < 0) {
+        if (LUKS_read_phdr(lukshdr, 1, 0, luks_ctxt) < 0) {
             fprintf(stderr, _("Failed to read LUKS header for \"%s\"\n"),
                     boundtgt->tgt->ident);
             eflag = ERR_BADDEVICE;
@@ -353,7 +356,7 @@ static int kmluks_put_key(bound_tgtdefn_t *boundtgt,
 
     printf(_("Setting password on LUKS keyslot-%u\n"), keyslot);
 
-    if (LUKS_set_key(tgtdev, keyslot, passwd, strlen(passwd), lukshdr, lukskey, *iteration_time, PBKDF2_per_sec, luks_ctxt) < 0) {
+    if (LUKS_set_key(keyslot, passwd, strlen(passwd), lukshdr, lukskey, *iteration_time, PBKDF2_per_sec, luks_ctxt) < 0) {
         char errmsg[80];
         fprintf(stderr, _("Failed to create LUKS key for \"%s\"\n"),
                 boundtgt->tgt->ident);
@@ -469,6 +472,8 @@ keymanager_t *kmluks_gethandle()
 
 #endif  /* !AS_MODULE */
 
+/**  @} */
+
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
