@@ -1,6 +1,6 @@
 /*
  *  Declarations for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -26,6 +26,9 @@
 
 #include "cryptmount.h"
 
+/*! \addtogroup keymgrs
+ *  @{ */
+
 
 struct keyinfo;
 struct bound_tgtdefn;
@@ -33,7 +36,7 @@ struct cm_testinfo;
 struct km_pw_context;
 struct km_overrides;
 
-/* Abstract interface to manager of filesystem access keys: */
+/*! Abstract interface to manager of filesystem access keys: */
 typedef struct keymanager {
     const char *ident;
 
@@ -41,24 +44,24 @@ typedef struct keymanager {
     int (*init_algs)(void);
     int (*free_algs)(void);
 
-    /*  Attempt to attach to particular target,
+    /*! Attempt to attach to particular target,
         installing default fields in target-definition */
     int (*bind)(struct bound_tgtdefn *boundtgt, FILE *fp_key);
 
-    /* Get properties, e.g. whether a password is needed for access: */
+    /*! Get properties, e.g. whether a password is needed for access: */
     unsigned (*get_properties)(const struct bound_tgtdefn *boundtgt);
 
-    /* Extract encrypted key from file: */
+    /*! Extract encrypted key from file: */
     int (*get_key)(struct bound_tgtdefn *boundtgt,
                     const struct km_pw_context *pw_ctxt,
                     unsigned char **key, int *keylen, FILE *fp_key);
 
-    /* Write encrypted key into file: */
+    /*! Write encrypted key into file: */
     int (*put_key)(struct bound_tgtdefn *boundtgt,
                     const struct km_pw_context *pw_ctxt,
                     const unsigned char *key, const int keylen, FILE *fp_key);
 
-    /* Linked-list scaffolding: */
+    /*! Linked-list scaffolding: */
     struct keymanager *next;
 
 #ifdef TESTING
@@ -68,22 +71,22 @@ typedef struct keymanager {
 #endif
 } keymanager_t;
 
-/* Key-manager initialization status flags: */
+/*! Key-manager initialization status flags: */
 enum {
     KM_INIT_ALGS =          0x001,
     KM_TESTED =             0x800
 };
 
-/* Key-manager key-properties flags: */
+/*! Key-manager key-properties flags: */
 enum {
-    KM_PROP_HASPASSWD =     0x001,      /* Password needed to access key */
-    KM_PROP_NEEDSKEYFILE =  0x002,      /* Key-file must be present */
-    KM_PROP_FIXEDLOC =      0x004,      /* Key-file cannot be renamed */
-    KM_PROP_FORMATTED =     0x008       /* Key-file has been formatted */
+    KM_PROP_HASPASSWD =     0x001,      /*!< Password needed to access key */
+    KM_PROP_NEEDSKEYFILE =  0x002,      /*!< Key-file must be present */
+    KM_PROP_FIXEDLOC =      0x004,      /*!< Key-file cannot be renamed */
+    KM_PROP_FORMATTED =     0x008       /*!< Key-file has been formatted */
 };
 
 
-/* Association of user-defined target-data & particular key-manager: */
+/*! Association of user-defined target-data & particular key-manager: */
 typedef struct bound_tgtdefn
 {
     tgtdefn_t *tgt;
@@ -131,9 +134,10 @@ int cm_mutex_unlock(void);
     table = dlsym(dlptr, SYMBOL);                                           \
     return (keymanager_t*)table;
 
+/**  @} */
 
 #endif  /* _ARMOUR_H */
 
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
