@@ -1,6 +1,6 @@
 /*
  *  cryptmount - a utility for user-level mounting of encrypted filesystems
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -59,8 +59,11 @@
 #  include "cmtesting.h"
 #endif
 
-
-typedef struct targelt          /* Element of filesystem-target list */
+/**
+ *  An element within a linked-list of filesystem-targets,
+ *  typically supplied from command-line.
+ */
+typedef struct targelt
 {
     const tgtdefn_t *tgt;
 
@@ -68,23 +71,38 @@ typedef struct targelt          /* Element of filesystem-target list */
 } targelt_t;
 
 
-typedef enum                    /* Top-level operating mode */
-{
-    M_DEFAULT, M_HELP,
+/** Identifiers of top-level operating mode */
+typedef enum {
+    M_UNSET, M_HELP,
     M_PREPARE, M_RELEASE,
     M_MOUNT, M_UNMOUNT,
     M_SWAPON, M_SWAPOFF,
     M_LIST, M_KEYMGRS,
     M_PASSWORD, M_KEYGEN, M_KEYREU,
-    M_SAFETYNET, M_VERSION
+    M_SYSBOOT, M_SYSQUIT,
+    M_SAFETYNET, M_VERSION,
+    M_SELFTEST
 } cmmode_t;
 
+/**
+ *  Configuration switches set (perhaps indirectly)
+ *  from command-line options
+ */
+enum {
+    F_NEEDS_TGT =   1,
+    F_ALL_TARGETS = 2,
+    F_VERIFY_PW =   4
+};
+
 
 static int64_t getblk512count(const char *device, int *blklen);
 static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
                 const km_pw_context_t *pw_ctxt,
                 const char *params, const targelt_t *eltlist);
-static int do_list(const tgtdefn_t *tgt);
+static int do_list(const targelt_t *eltlist);
+static int do_keymgrlist();
+static int do_sysboot_updown(int booting, const tgtdefn_t *tgttable,
+                const km_pw_context_t *pw_ctxt);
 static int do_devsetup(const km_pw_context_t *pw_ctxt,
                 bound_tgtdefn_t *boundtgt, char **mntdev);
 static int do_devshutdown(const bound_tgtdefn_t *boundtgt);
@@ -127,7 +145,7 @@ usage: cryptmount [OPTION [target ...]]\n\
 #ifdef TESTING
 
 cm_testinfo_t test_context;
-cm_testinfo_t *test_ctxtptr=&test_context;
+cm_testinfo_t *test_ctxtptr = &test_context;
 
 int fs_test_blkgetsz()
     /** Check that 32bit & 64bit device size calculations agree */
@@ -216,21 +234,87 @@ static int64_t getblk512count(const char *device, int *blklen)
 }
 
 
-static int do_list(const tgtdefn_t *tgt)
-    /** Print list of available filing-system targets */
-{
-    /* TRANSLATORS: this string is marked as 'no-c-format' because
-       some localizations may require the mount-point and filesystem type
-       to be printed in a different order, but the untranslated string needs
-       to remain an ordinary string that can be printed without gettext. */
-    /* xgettext:no-c-format */
-    printf(_("%-16s  [to mount on \"%s\" as \"%s\"]\n"),
-            tgt->ident, tgt->dir, tgt->fstype);
+static int do_list(const targelt_t *eltlist)
+    /** Print list of available filing-system targets to stdout */
+{   const targelt_t *elt;
+
+    for (elt=eltlist; elt!=NULL; elt=elt->nx) {
+        const tgtdefn_t *tgt = elt->tgt;
+
+        /* TRANSLATORS: this string is marked as 'no-c-format' because
+           some localizations may require the mount-point and filesystem type
+           to be printed in a different order, but the untranslated string needs
+           to remain an ordinary string that can be printed without gettext. */
+        /* xgettext:no-c-format */
+        printf(_("%-16s  [to mount on \"%s\" as \"%s\"]\n"),
+                tgt->ident, tgt->dir, tgt->fstype);
+    }
 
     return ERR_NOERROR;
 }
 
 
+/**
+ *  Print a list of all available key managers to stdout.
+ */
+static int do_keymgrlist()
+{   const char **keymgrs = get_keymgr_list();
+    int i = 0;
+
+    while (keymgrs[i] != NULL) {
+        printf("%s", keymgrs[i]);
+
+        ++i;
+        if (keymgrs[i] != NULL) {
+            printf(", ");
+        } else {
+            printf("\n");
+        }
+    }
+
+    free((void*)keymgrs);
+    return ERR_NOERROR;
+}
+
+
+static int do_sysboot_updown(int booting, const tgtdefn_t *tgttable,
+                            const km_pw_context_t *pw_ctxt)
+    /** Setup all devices which have a bootaction option specified */
+{   const tgtdefn_t *tgt;
+    bound_tgtdefn_t *boundtgt = NULL;
+    int eflag = ERR_NOERROR;
+
+    for (tgt=tgttable; tgt!=NULL && eflag<ERR_threshold; tgt=tgt->nx) {
+        const unsigned boot_mode = (tgt->flags & FLG_BOOT_MASK);
+
+        if (boot_mode == 0) continue;
+        boundtgt = bind_tgtdefn(tgt);
+        if (boundtgt == NULL) continue;
+
+        switch (boot_mode) {
+            case FLG_BOOT_MOUNT:
+                eflag = (booting ? do_mount(pw_ctxt, boundtgt)
+                                : do_unmount(boundtgt));
+                break;
+            case FLG_BOOT_SWAP:
+                eflag = (booting ? do_swapon(pw_ctxt, boundtgt)
+                                : do_swapoff(boundtgt));
+                break;
+            case FLG_BOOT_PREP:
+                eflag = (booting ? do_devsetup(pw_ctxt, boundtgt, NULL)
+                                : do_devshutdown(boundtgt));
+                break;
+            default:
+                break;
+        }
+
+        free_boundtgt(boundtgt);
+    }
+
+    return eflag;
+}
+
+
 static int do_devsetup(const km_pw_context_t *pw_ctxt,
                     bound_tgtdefn_t *boundtgt, char **mntdev)
     /** Setup all devices needed to access encrypted target */
@@ -689,7 +773,7 @@ static int do_keygen(const km_pw_context_t *pw_ctxt, bound_tgtdefn_t *boundtgt,
     } else {
         fprintf(stderr, _("Generating random key; please be patient...\n"));
         key = (unsigned char*)sec_realloc(NULL, (size_t)keylen);
-        eflag = get_randkey(key, (size_t)keylen);
+        eflag = cm_generate_key(key, (size_t)keylen);
         if (eflag != ERR_NOERROR) {
             fprintf(stderr, _("Failed to generate new key\n"));
             goto bail_out;
@@ -836,7 +920,6 @@ static int do_safetynet()
 static void check_priv_opt(const char *opt)
     /** Check if ordinary user is allowed to perform privileged actions */
 {
-
     if (getuid() != 0) {
         fprintf(stderr, _("Only root can use option \"%s\"\n"), opt);
         exit(EXIT_PRIV);
@@ -859,70 +942,15 @@ static int check_priv_tgt(const tgtdefn_t *tgt)
 }
 
 
-static int check_mode(cmmode_t mode, const char *params,
-                    const targelt_t *eltlist)
-    /** Check that arguments supplied & privilege match selected mode */
-{   enum {
-        F_NORMALUSER =  0x001,      /* normal user can perform action */
-        F_NEEDS_TGT =   0x010,      /* filesystem-target is required */
-        F_NEEDS_PRM =   0x020       /* additional parameter is required */
-    };
-    struct tblent {
-        cmmode_t mode;
-        unsigned flags; } *mdent=NULL;
-    struct tblent mdtbl[] = {
-        { M_HELP,       F_NORMALUSER },
-        { M_PREPARE,    F_NEEDS_TGT },
-        { M_RELEASE,    F_NEEDS_TGT },
-        { M_MOUNT,      F_NORMALUSER | F_NEEDS_TGT },
-        { M_UNMOUNT,    F_NORMALUSER | F_NEEDS_TGT },
-        { M_SWAPON,     F_NEEDS_TGT },
-        { M_SWAPOFF,    F_NEEDS_TGT },
-        { M_LIST,       F_NORMALUSER },
-        { M_KEYMGRS,    F_NORMALUSER },
-        { M_PASSWORD,   F_NORMALUSER | F_NEEDS_TGT },
-        { M_KEYGEN,     F_NEEDS_TGT | F_NEEDS_PRM },
-        { M_KEYREU,     F_NEEDS_TGT | F_NEEDS_PRM },
-        { M_SAFETYNET,  0u },
-        { M_VERSION,    F_NORMALUSER },
-    };
-    unsigned idx,nmds;
-
-    nmds = sizeof(mdtbl) / sizeof(mdtbl[0]);
-    for (mdent=mdtbl,idx=0; idx<nmds && mdent->mode!=mode; ++mdent,++idx);
-    if (idx >= nmds) {
-        fprintf(stderr, "Bad mode [%u]\n", mode);
-        abort();
-    }
-
-    if ((mdent->flags & F_NORMALUSER) != F_NORMALUSER) {
-        /* turn off any setuid effects for privileged modes: */
-        seteuid(getuid());
-    }
-
-    if ((mdent->flags & F_NEEDS_TGT) == F_NEEDS_TGT && eltlist == NULL) {
-        fprintf(stderr, _("No targets specified\n"));
-        exit(EXIT_BADTGT);
-    }
-
-    if ((mdent->flags & F_NEEDS_PRM) == F_NEEDS_PRM && params == NULL) {
-        fprintf(stderr, _("Missing parameter\n"));
-        exit(EXIT_BADOPT);
-    }
-
-    return 0;
-}
-
-
 static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
                 const km_pw_context_t *pw_ctxt,
                 const char *params, const targelt_t *eltlist)
     /** Apply mode-dependent operation to list of targets */
-{   const targelt_t *elt=NULL;
-    bound_tgtdefn_t *boundtgt=NULL;
-    int i, prio=0, eflag=ERR_NOERROR;
-    struct passwd *pwent=NULL;
-    const char *username=NULL, **keymgrs=NULL, *syslogmsg=NULL;
+{   const targelt_t *elt = NULL;
+    bound_tgtdefn_t *boundtgt = NULL;
+    int ignore_eltlist = 1, prio = 0, eflag = ERR_NOERROR;
+    struct passwd *pwent = NULL;
+    const char *username = NULL, *syslogmsg = NULL;
 
     pwent = getpwuid(getuid());
     username = (pwent != NULL ? pwent->pw_name : "UNKNOWN");
@@ -931,30 +959,31 @@ static int execute_list(cmmode_t mode, const tgtdefn_t *tgttable,
     openlog(PACKAGE, LOG_PID, LOG_AUTHPRIV);
 #endif
 
-    /* Check that we have suitable privileges + parameters: */
-    (void)check_mode(mode, params, eltlist);
-
     /* Execute special-cases of user-selected task: */
-    if (mode == M_VERSION) {
-        fprintf(stderr, "%s-%s\n", PACKAGE_NAME, PACKAGE_VERSION);
-        eltlist = NULL;
-    } else if (mode == M_KEYMGRS) {
-        keymgrs = get_keymgr_list();
-        for (i=0; keymgrs[i]!=NULL; ) {
-            printf("%s", keymgrs[i]);
-            if (keymgrs[++i] != NULL) printf(", "); else printf("\n");
-        }
-        free((void*)keymgrs);
-        eltlist = NULL;
-    } else if (mode == M_LIST) {
-        for (elt=eltlist; elt!=NULL; elt=elt->nx) {
-            (void)do_list(elt->tgt);
-        }
-        eltlist = NULL;
-    } else if (mode == M_SAFETYNET) {
-        do_safetynet();
-        eltlist = NULL;
+    switch (mode) {
+        case M_VERSION:
+            fprintf(stderr, "%s-%s\n", PACKAGE_NAME, PACKAGE_VERSION);
+            break;
+        case M_KEYMGRS:
+            do_keymgrlist();
+            break;
+        case M_LIST:
+            do_list(eltlist);
+            break;
+        case M_SYSBOOT:
+            do_sysboot_updown(1, tgttable, pw_ctxt);
+            break;
+        case M_SYSQUIT:
+            do_sysboot_updown(0, tgttable, pw_ctxt);
+            break;
+        case M_SAFETYNET:
+            do_safetynet();
+            break;
+        default:
+            ignore_eltlist = 0;
+            break;
     }
+    if (ignore_eltlist) eltlist = NULL;
 
     /* Apply user-selected operation to list of targets (if present): */
     for (elt=eltlist; elt!=NULL && eflag<ERR_threshold; elt=elt->nx) {
@@ -1051,7 +1080,7 @@ static cmmode_t get_defaultmode(int argc, char *argv[])
 #endif
             { M_PREPARE,    "cryptprepare" },
             { M_RELEASE,    "cryptrelease" },
-            { M_DEFAULT, NULL } },
+            { M_MOUNT, NULL } },
         *mp;
     const char *base;
 
@@ -1073,46 +1102,181 @@ static cmmode_t get_defaultmode(int argc, char *argv[])
 }
 
 
-int main(int argc, char *argv[])
-{   cmmode_t mode=M_DEFAULT;
+/**
+ *  Parse command-line options to identify a top-level processing mode,
+ *  and extract configuration parameters such as key-length,
+ *  password source etc.
+ *  This routine will also ensure that 'privileged' options are
+ *  only accessible to the super-user.
+ */
+cmmode_t parse_options(int argc, char *argv[],
+                        unsigned *flags, const char **mode_params,
+                        int *passwd_fd, int *config_fd,
+                        km_pw_context_t *pw_ctxt)
+{   cmmode_t mode = M_UNSET;
     enum {
-        F_ALL=0x1 }
-        flags=0;
-#ifdef _GNU_SOURCE
-    struct option opttable[] = {
-        { "all",                no_argument,        NULL, (int)'a' },
-        { "change-password",    no_argument,        NULL, (int)'c' },
-        { "config-fd",          required_argument,  NULL, (int)'f' },
-        { "generate-key",       required_argument,  NULL, (int)'g' },
-        { "help",               no_argument,        NULL, (int)'h' },
-        { "key-managers",       no_argument,        NULL, (int)'k' },
-        { "list",               no_argument,        NULL, (int)'l' },
-        { "mount",              no_argument,        NULL, (int)'m' },
-        { "passwd-fd",          required_argument,  NULL, (int)'w' },
-        { "prepare",            no_argument,        NULL, (int)'p' },
-        { "release",            no_argument,        NULL, (int)'r' },
-        { "reuse-key",          required_argument,  NULL, (int)'e' },
-        { "safetynet",          no_argument,        NULL, (int)'n' },
-        { "swapon",             no_argument,        NULL, (int)'s' },
-        { "swapoff",            no_argument,        NULL, (int)'x' },
-        { "unmount",            no_argument,        NULL, (int)'u' },
-        { "verify-password",    no_argument,        NULL, (int)'y' },
-        { "version",            no_argument,        NULL, (int)'v' },
-#  ifdef TESTING
-        { "config-dir",         required_argument,  NULL, (int)'D' },
-        { "password",           required_argument,  NULL, (int)'W' },
-        { "newpassword",        required_argument,  NULL, (int)'N' },
-        { "self-test",          no_argument,        NULL, (int)'T' },
-#  endif
-        { NULL, 0, NULL, 0 } };
+        ALL_USERS =     0x01,
+        NEEDS_ARG =     0x02,
+        SET_MODE =      0x04,
+        SET_FLAGS =     0x08,
+        NEEDS_TGT =     0x10
+    };
+    const char *passwd_fd_str = NULL, *config_fd_str = NULL;
+    struct cm_option {
+        char shortopt;
+        const char *longopt;
+        unsigned flags;
+        const char **argument;
+        unsigned newmode;
+        unsigned newflags;
+    };
+    struct cm_option opt_table[] = {
+        { 'a', "all",               ALL_USERS | SET_FLAGS,
+                                    NULL, 0, F_ALL_TARGETS },
+        { 'c', "change-password",   ALL_USERS | SET_MODE | SET_FLAGS,
+                                    NULL, M_PASSWORD, F_NEEDS_TGT },
+        { 'f', "config-fd",         NEEDS_ARG,
+                                    &config_fd_str, 0, 0 },
+        { 'g', "generate-key",      NEEDS_ARG | SET_MODE | SET_FLAGS,
+                                    mode_params, M_KEYGEN, F_NEEDS_TGT },
+        { 'h', "help",              ALL_USERS | SET_MODE,
+                                    NULL, M_HELP, 0 },
+        { 'k', "key-managers",      ALL_USERS | SET_MODE,
+                                    NULL, M_KEYMGRS, 0 },
+        { 'l', "list",              ALL_USERS | SET_MODE,
+                                    NULL, M_LIST, 0 },
+        { 'm', "mount",             ALL_USERS | SET_MODE | SET_FLAGS,
+                                    NULL, M_MOUNT, F_NEEDS_TGT },
+        { 'w', "passwd-fd",         ALL_USERS | NEEDS_ARG,
+                                    &passwd_fd_str, 0, 0 },
+        { 'p', "prepare",           SET_MODE | SET_FLAGS,
+                                    NULL, M_PREPARE, F_NEEDS_TGT },
+        { 'r', "release",           SET_MODE | SET_FLAGS,
+                                    NULL, M_RELEASE, F_NEEDS_TGT },
+        { 'e', "reuse-key",         NEEDS_ARG | SET_MODE | SET_FLAGS,
+                                    mode_params, M_KEYREU, F_NEEDS_TGT },
+        { 'n', "safetynet",         SET_MODE,
+                                    NULL, M_SAFETYNET, 0 },
+        { 's', "swapon",            SET_MODE | SET_FLAGS,
+                                    NULL, M_SWAPON, F_NEEDS_TGT },
+        { 'x', "swapoff",           SET_MODE | SET_FLAGS,
+                                    NULL, M_SWAPOFF, F_NEEDS_TGT },
+        { 'B', "system-boot",       SET_MODE,
+                                    NULL, M_SYSBOOT, 0 },
+        { 'Q', "system-shutdown",   SET_MODE,
+                                    NULL, M_SYSQUIT, 0 },
+        { 'u', "unmount",           ALL_USERS | SET_MODE | SET_FLAGS,
+                                    NULL, M_UNMOUNT, F_NEEDS_TGT },
+        { 'y', "verify-password",   ALL_USERS | SET_FLAGS,
+                                    NULL, 0, F_VERIFY_PW },  /* FIXME - not implemented? */
+        { 'v', "version",           ALL_USERS | SET_MODE,
+                                    NULL, M_VERSION, 0 },
+#ifdef TESTING
+        { 'D', "config-dir",        ALL_USERS | NEEDS_ARG,
+                                    &test_context.argconfigdir, 0, 0 },
+        { 'W', "password",          ALL_USERS | NEEDS_ARG,
+                                    &pw_ctxt->argpasswd[0], 0, 0 },
+        { 'N', "newpassword",       ALL_USERS | NEEDS_ARG,
+                                    &pw_ctxt->argpasswd[1], 0, 0 },
+        { 'T', "self-test",         ALL_USERS | SET_MODE,
+                                    NULL, M_SELFTEST, 0 },
 #endif
-    const char *params=NULL, *optlist="acde:f:g:hklmnprsuvw:xy";
-    char *cmtab=NULL;
-    int val, idx, config_fd=-1, passwd_fd=-1, eflag=ERR_NOERROR;
+        { '?', NULL, 0, NULL, 0, 0 } };
+    const size_t n_options = (sizeof(opt_table) / sizeof(opt_table[0]) - 1);
+    char *shortopts, *spos;
+    int i, idx = 0, optchar = '\0';
+
+    shortopts = (char*)malloc(n_options * 2 + 1);
+    spos = shortopts;
+    for (i=0; i<n_options; ++i) {
+        *spos = opt_table[i].shortopt;
+        if ((opt_table[i].flags & NEEDS_ARG)) {
+            *++spos = ':';
+        }
+        ++spos;
+    }
+    *spos = '\0';
+
+#ifdef _GNU_SOURCE
+    struct option *longopts;
+
+    longopts = (struct option*)malloc((n_options + 1) * sizeof(struct option));
+    for (i=0; i<n_options; ++i) {
+        longopts[i].name =      opt_table[i].longopt;
+        longopts[i].has_arg =   ((opt_table[i].flags & NEEDS_ARG)
+                                    ? required_argument : no_argument);
+        longopts[i].flag =      NULL;
+        longopts[i].val =       (int)opt_table[i].shortopt;
+    }
+#endif  /* _GNU_SOURCE */
+
+    for (;;) {
+        struct cm_option *selected;
+#ifdef _GNU_SOURCE
+        optchar = getopt_long(argc, argv, shortopts, longopts, &idx);
+#else
+        optchar = getopt(argc, argv, shortopts);
+#endif
+        if (optchar < 0 || optchar == '?') break;
+        idx = 0;
+        while (opt_table[idx].shortopt != optchar
+                && opt_table[idx].longopt != NULL) ++idx;
+        if (opt_table[idx].longopt == NULL) {
+            optchar = '?';
+            break;
+        }
+        selected = opt_table + idx;
+
+        if (!(selected->flags & ALL_USERS)) {
+            check_priv_opt(selected->longopt);
+        }
+
+        if ((selected->flags & NEEDS_ARG)) {
+            *selected->argument = optarg;
+        }
+
+        if ((selected->flags & SET_MODE)) {
+            if (mode == M_UNSET) {
+                mode = selected->newmode;
+            } else {
+                fprintf(stderr, _("Multiple operating modes not supported\n"));
+                exit(1);
+            }
+        }
+
+        if ((selected->flags & SET_FLAGS)) {
+            *flags |= selected->newflags;
+        }
+    }
+
+    if (optchar == '?') {
+        fprintf(stderr, "%s", _(USAGE_STRING));
+        exit(EXIT_BADOPT);
+    }
+
+    if (mode == M_UNSET) mode = get_defaultmode(argc, argv);
+    if (config_fd_str != NULL) sscanf(config_fd_str, "%d", config_fd);
+    if (passwd_fd_str != NULL) sscanf(passwd_fd_str, "%d", passwd_fd);
+
+#ifdef _GNU_SOURCE
+    free((void*)longopts);
+#endif
+    free((void*)shortopts);
+
+    return mode;
+}
+
+
+int main(int argc, char *argv[])
+{   cmmode_t mode = M_UNSET;
+    unsigned mode_flags = 0;
+    const char *mode_params = NULL;
+    char *cmtab = NULL;
+    int config_fd = -1, passwd_fd = -1, eflag = ERR_NOERROR;
     tgtdefn_t *tgttable=NULL;
     km_pw_context_t pw_ctxt;
-    const tgtdefn_t *tgt=NULL;
-    targelt_t *eltlist=NULL, **eltptr=&eltlist;
+    const tgtdefn_t *tgt = NULL;
+    targelt_t *eltlist = NULL, **eltptr = &eltlist;
 
 #ifdef HAVE_GETTEXT
     /* setup internationalization of message-strings via gettext(): */
@@ -1136,74 +1300,19 @@ int main(int argc, char *argv[])
     pw_ctxt.verify = 0;
     pw_ctxt.debug_level = 0;
 
-    /* Parse command-line options: */
-    for (;;) {
-#ifdef _GNU_SOURCE
-        val = getopt_long(argc, argv, optlist, opttable, &idx);
-#else
-        val = getopt(argc, argv, optlist);
-#endif
-        if (val == -1) break;
-        switch (val) {
-            case 'a':
-                flags |= F_ALL; break;
-            case 'c':
-                mode = M_PASSWORD; break;
-            case 'e':                       check_priv_opt("--reuse-key");
-                mode = M_KEYREU; params = optarg; break;
-            case 'f':                       check_priv_opt("--config-fd");
-                sscanf(optarg, "%d", &config_fd); break;
-            case 'g':                       check_priv_opt("--generate-key");
-                mode = M_KEYGEN; params = optarg; break;
-            case 'h':
-                mode = M_HELP; break;
-            case 'k':
-                mode = M_KEYMGRS; break;
-            case 'l':
-                mode = M_LIST;  break;
-            case 'm':
-                mode = M_MOUNT; break;
-            case 'n':                       check_priv_opt("--safetynet");
-                mode = M_SAFETYNET; break;
-            case 'p':                       check_priv_opt("--prepare");
-                mode = M_PREPARE; break;
-            case 'r':                       check_priv_opt("--release");
-                mode = M_RELEASE; break;
-            case 's':                       check_priv_opt("--swapon");
-                mode = M_SWAPON; break;
-            case 'x':                       check_priv_opt("--swapoff");
-                mode = M_SWAPOFF; break;
-            case 'u':
-                mode = M_UNMOUNT; break;
-            case 'v':
-                mode = M_VERSION; break;
-            case 'w':
-                sscanf(optarg, "%d", &passwd_fd); break;
+    mode = parse_options(argc, argv, &mode_flags, &mode_params,
+                        &passwd_fd, &config_fd, &pw_ctxt);
+
 #ifdef TESTING
-            case 'D':
-                test_context.argconfigdir = optarg; break;
-            case 'W':
-                pw_ctxt.argpasswd[0] = optarg; break;
-            case 'N':
-                pw_ctxt.argpasswd[1] = optarg; break;
-            case 'T':
-                eflag = cm_run_tests();
-                clear_env_dictionary();
-                return eflag; break;
-#endif
-            case '?':
-                fprintf(stderr, "%s", _(USAGE_STRING));
-                exit(EXIT_BADOPT);
-                break;
-            default:
-                break;
-        }
+    if (mode == M_SELFTEST) {
+        eflag = cm_run_tests();
+        clear_env_dictionary();
+        return eflag;
     }
+#endif
 
     (void)cm_path(&cmtab, "cmtab");
 
-    if (mode == M_DEFAULT) mode = get_defaultmode(argc, argv);
-
     if (mode == M_HELP) {
         fprintf(stderr, "%s", _(USAGE_STRING));
         exit(EXIT_OK);
@@ -1235,10 +1344,10 @@ int main(int argc, char *argv[])
     }
     free((void*)cmtab);
 
-    if (mode == M_LIST && optind >= argc) flags |= F_ALL;
+    if (mode == M_LIST && optind >= argc) mode_flags |= F_ALL_TARGETS;
 
     /* if '--all' given, assemble list of targets from entire config-file */
-    if ((flags & F_ALL) != 0) {
+    if ((mode_flags & F_ALL_TARGETS) != 0) {
         if (optind < argc) {
             fprintf(stderr, _("Trailing command-line arguments given with '--all' option\n"));
             exit(EXIT_BADOPT);
@@ -1280,7 +1389,11 @@ int main(int argc, char *argv[])
 
 
     /* Execute user-selected task: */
-    eflag = execute_list(mode, tgttable, &pw_ctxt, params, eltlist);
+    if ((mode_flags & F_NEEDS_TGT) && eltlist == NULL) {
+        fprintf(stderr, _("No targets specified\n"));
+        exit(EXIT_BADTGT);
+    }
+    eflag = execute_list(mode, tgttable, &pw_ctxt, mode_params, eltlist);
     free_keymanagers();
 
 
@@ -1299,5 +1412,5 @@ int main(int argc, char *argv[])
 }
 
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
