@@ -1,6 +1,6 @@
 /*
  *  General declarations for cryptmount
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -56,7 +56,7 @@
 #endif
 
 
-enum    /* Exit-codes */
+enum    /*! Exit-codes */
 {
     EXIT_OK =       0,
     EXIT_BADOPT =   1,
@@ -67,7 +67,7 @@ enum    /* Exit-codes */
 };
 
 
-enum    /* Error flags */
+enum    /*! Error flags */
 {
     ERR_NOERROR =       0,
 
@@ -75,13 +75,13 @@ enum    /* Error flags */
     WRN_NOPASSWD,
     WRN_LOWENTROPY,
 
-    ERR_threshold = 0x10,       /* Dividing-line between warnings & errors */
+    ERR_threshold = 0x10,       /*!< Dividing-line between warnings & errors */
 
     ERR_NOTSUPPORTED,
     ERR_BADKEYFORMAT,
     ERR_BADALGORITHM,
-    ERR_BADFILE,                /* Serious problem with accessing file */
-    ERR_BADDECRYPT,             /* Failure to extract cipher key from file */
+    ERR_BADFILE,                /*!< Serious problem with accessing file */
+    ERR_BADDECRYPT,             /*!< Failure to extract cipher key from file */
     ERR_BADENCRYPT,
     ERR_MEMSPACE,
     ERR_DMSETUP,
@@ -100,11 +100,16 @@ enum    /* Error flags */
 };
 
 
-enum    /* Target configuration switches */
+enum    /*! Target configuration switches */
 {
-    FLG_USER =      0x001,
-    FLG_FSCK =      0x002,
-    FLG_MKSWAP =    0x004,
+    FLG_USER =          0x0001,
+    FLG_FSCK =          0x0002,
+    FLG_MKSWAP =        0x0004,
+
+    FLG_BOOT_MASK =     0xf000,
+    FLG_BOOT_MOUNT =    0x1000,
+    FLG_BOOT_SWAP =     0x2000,
+    FLG_BOOT_PREP =     0x3000,
 
     FLG_DEFAULTS =  FLG_USER | FLG_FSCK
 };
@@ -112,38 +117,45 @@ enum    /* Target configuration switches */
 
 typedef struct keyinfo
 {
-    const char *format;     /* Type of key file, e.g. 'raw', 'libgcrypt' */
+    const char *format;     /*!< Type of key file, e.g. 'raw', 'libgcrypt' */
     char *filename;
     char *digestalg;
     char *cipheralg;
-    long maxlen;            /* Maximum number of bytes to read from keyfile */
-    unsigned retries;       /* Limit on password-entry attempts */
+    long maxlen;            /*!< Maximum number of bytes to read from keyfile */
+    unsigned retries;       /*!< Limit on password-entry attempts */
 } keyinfo_t;
 
 
-typedef struct tgtdefn      /* Entry in list of cryptmount filesystem-targets */
+/*!
+ *  Description of an available encrypted filesystem or device,
+ *  typically as extracted from a configuration file, containing
+ *  details of its name, underlying device, encryption type etc.
+ *  This structure can be used to form a linked-list of
+ *  cryptmount filesystem-targets.
+ */
+typedef struct tgtdefn
 {
-    const char *ident;      /* Unique identifying name */
-    unsigned flags;         /* Configuration switches */
+    const char *ident;      /*!< Unique identifying name */
+    unsigned flags;         /*!< Configuration switches */
 
-    char *dev;              /* Device node or raw file */
-    int64_t start, length;  /* Starting sector + num of sectors (or 0, -1) */
-    char *dir;              /* Mount-point */
-    char *fstype;           /* Filesystem type */
-    char *mountoptions;     /* Options passed to 'mount' command */
-    char *fsckoptions;      /* Options passed to 'fsck' command */
-    char *loopdev;          /* Loopback device to wrap around raw file */
+    char *dev;              /*!< Device node or raw file */
+    int64_t start, length;  /*!< Starting sector + num of sectors (or 0, -1) */
+    char *dir;              /*!< Mount-point */
+    char *fstype;           /*!< Filesystem type */
+    char *mountoptions;     /*!< Options passed to 'mount' command */
+    char *fsckoptions;      /*!< Options passed to 'fsck' command */
+    char *loopdev;          /*!< Loopback device to wrap around raw file */
 
-    char *cipher;           /* Cipher used on filesystem */
-    int64_t ivoffset;       /* Cipher initialization-vector offset */
+    char *cipher;           /*!< Cipher used on filesystem */
+    int64_t ivoffset;       /*!< Cipher initialization-vector offset */
 
-    keyinfo_t key;          /* Location/format of key */
+    keyinfo_t key;          /*!< Location/format of key */
 
-    struct tgtdefn *nx;       /* Form into linked list */
+    struct tgtdefn *nx;     /*!< Form into linked list */
 } tgtdefn_t;
 
 #endif  /* _CRYPTMOUNT_H */
 
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
