@@ -1,6 +1,6 @@
 /*
  *  Config-table and mount-table utilities for cryptmount
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -51,6 +51,17 @@ enum                /* config-file tokens */
 };
 
 
+/**
+ *  Description of an open target-status file.
+ *
+ *  Internally, the status-file has the following structure:
+ \verbatim
+    # [TITLE]           - one-line text header
+    0                   - integer file-format version number
+    n,[TARGET],uid      - target-name length, target-name, uid of mounter
+    ...                 - further target records
+ \endverbatim
+ */
 struct statfile     /* information about status file */
 {
     int version;        /* file format version */
@@ -58,15 +69,39 @@ struct statfile     /* information about status file */
 };
 
 
-/*  The format for the status-file (version 0) is as follows:
-    # [TITLE]           - one-line text header
-    0                   - integer file-format version number
-    n,[TARGET],uid      - target-name length, target-name, uid of mounter
-    ...                 - further target records
-*/
+/**
+ *  Representation of an option flag within the configuration file,
+ *  associated with a bit field within a target definition.
+ */
+typedef struct {
+    const char *name;
+    unsigned andmask;
+    unsigned ormask;
+} tgt_option_t;
 
 
+static tgt_option_t
+    /** Menu of options within 'flags' field of target definition */
+    setup_options[] = {
+        { "defaults",   0U,             FLG_DEFAULTS },
+        { "user",       ~0U,            FLG_USER },
+        { "nouser",     ~FLG_USER,      0U },
+        { "fsck",       ~0U,            FLG_FSCK },
+        { "nofsck",     ~FLG_FSCK,      0U },
+        { "mkswap",     ~0U,            FLG_MKSWAP },
+        { "nomkswap",   ~FLG_MKSWAP,    0U },
+        { NULL, ~0U, 0U } },
+    /** Menu of options within 'bootaction' field of target definition */
+    boot_options[] = {
+        { "none",       ~FLG_BOOT_MASK,     0 },
+        { "mount",      ~FLG_BOOT_MASK,     FLG_BOOT_MOUNT },
+        { "swap",       ~FLG_BOOT_MASK,     FLG_BOOT_SWAP },
+        { "prepare",    ~FLG_BOOT_MASK,     FLG_BOOT_PREP },
+        { NULL, ~0U, 0U }
+    };
+
 struct vardefn
+    /** Description of a quasi-environmental variable */
 {
     char *varname;
     char *value;
@@ -79,6 +114,7 @@ static tgtdefn_t *parse_stream(FILE *fp, const char *cfgname);
 
 
 void set_variable(char *varname, char *value)
+    /** Set the value of an environmental variable */
 {   struct vardefn *newvar;
 
     newvar = (struct vardefn*)malloc(sizeof(struct vardefn));
@@ -92,6 +128,7 @@ void set_variable(char *varname, char *value)
 
 
 const char *query_variable(char *varname)
+    /** Get the current value of an environmental variable */
 {   struct vardefn *var;
 
     for (var=vardefns; var!=NULL; var=var->next) {
@@ -352,7 +389,7 @@ int tb_test_expand()
 #endif  /* TESTING */
 
 
-static int proc_string(void *ptr, const char *src)
+static int proc_string(void *ptr, const char *src, const void *config)
     /** Process var=val entry in config-table for "string" type */
 {   char **addr = (char**)ptr;
 
@@ -362,14 +399,14 @@ static int proc_string(void *ptr, const char *src)
 }
 
 
-#define PROC_VARVAL(FN_NAME, TYPE, FMT)                         \
-    static int FN_NAME(void *ptr, const char *src)              \
-        /** Process var=val entry in config-table for "TYPE" */ \
-    {   TYPE qv, *addr=(TYPE*)ptr;                              \
-        if (sscanf(src, FMT, &qv) == 1) {                       \
-            *addr = qv; return 0;                               \
-        }                                                       \
-        return 1;                                               \
+#define PROC_VARVAL(FN_NAME, TYPE, FMT)                             \
+    static int FN_NAME(void *ptr, const char *src, const void *x)   \
+        /** Process var=val entry in config-table for "TYPE" */     \
+    {   TYPE qv, *addr=(TYPE*)ptr;                                  \
+        if (sscanf(src, FMT, &qv) == 1) {                           \
+            *addr = qv; return 0;                                   \
+        }                                                           \
+        return 1;                                                   \
     }
 
 PROC_VARVAL(proc_unsigned, unsigned, "%u")
@@ -379,74 +416,89 @@ PROC_VARVAL(proc_int64, int64_t, "%" SCNi64)
 #undef PROC_VARVAL
 
 
-int proc_flags(void *ptr, const char *src)
-    /** Convert string of configuration-switches into binary flags */
-{   unsigned *flags=(unsigned*)ptr;
-    struct opt_t {
-        const char *str;
-        unsigned andmask, ormask; };
-    struct opt_t opts[] = {
-        { "defaults",   0U,             FLG_DEFAULTS },
-        { "user",       ~0U,            FLG_USER },
-        { "nouser",     ~FLG_USER,      0U },
-        { "fsck",       ~0U,            FLG_FSCK },
-        { "nofsck",     ~FLG_FSCK,      0U },
-        { "mkswap",     ~0U,            FLG_MKSWAP },
-        { "nomkswap",   ~FLG_MKSWAP,    0U },
-        { NULL, ~0U, 0U } };
-    unsigned idx,len;
+/**
+ *  Convert string of comma-separated configuration-switches
+ *  within \a selection into binary flags. A pointer to the (unsigned) flags,
+ *  is supplied via \a flagptr, and the set of available switches
+ *  is in the form of an array of tgt_option_t, passed via \a menuptr.
+ */
+int proc_flags(void *flagptr, const char *selection, const void *menuptr)
+{   unsigned *flags = (unsigned*)flagptr;
+    const tgt_option_t *menu = (const tgt_option_t*)menuptr;
+    unsigned idx, len;
 
-    if (src == NULL) return 0;
+    if (selection == NULL) return 0;
 
     for (;;) {
-        for (len=0; src[len]!='\0' && src[len]!=','; ++len);
-        for (idx=0; opts[idx].str!=NULL; ++idx) {
-            if (strncmp(src, opts[idx].str, (size_t)len) == 0) {
-                *flags = (*flags & opts[idx].andmask) | opts[idx].ormask;
+        for (len=0; selection[len]!='\0' && selection[len]!=','; ++len);
+        for (idx=0; menu[idx].name!=NULL; ++idx) {
+            if (strncmp(selection, menu[idx].name, (size_t)len) == 0) {
+                *flags = (*flags & menu[idx].andmask)
+                        | menu[idx].ormask;
                 break;
             }
         }
-        if (opts[idx].str == NULL) {
-            fprintf(stderr, "bad option \"%s\"\n", src);
+        if (menu[idx].name == NULL) {
+            fprintf(stderr, "bad option \"%s\"\n", selection);
             return 1;
         }
 
-        if (src[len] == '\0') break;
-        src += len + 1;
+        if (selection[len] == '\0') break;
+        selection += len + 1;
     }
 
     return 0;
 }
 
 
-
 static void read_token(char *buff, unsigned *t_state, tgtdefn_t *tgt)
     /** Process token (word) from configuration-file while parsing */
 {   struct tokinfo_t {
         const char *name;
         int varoffset;
-        int (*proc)(void *var, const char *val);
+        int (*proc)(void *var, const char *val, const void *config);
+        const void *config;
     } *tok;
 #define OFFSET(x) (int)((char*)&((tgtdefn_t*)NULL)->x - (char*)NULL)
     struct tokinfo_t toktable[] = {
-        { "flags",          OFFSET(flags),          proc_flags },
-        { "dev",            OFFSET(dev),            proc_string },
-        { "dir",            OFFSET(dir),            proc_string },
-        { "startsector",    OFFSET(start),          proc_int64 },
-        { "numsectors",     OFFSET(length),         proc_int64 },
-        { "fstype",         OFFSET(fstype),         proc_string },
-        { "mountoptions",   OFFSET(mountoptions),   proc_string },
-        { "fsoptions",      OFFSET(mountoptions),   proc_string },  /* Deprecated */
-        { "fsckoptions",    OFFSET(fsckoptions),    proc_string },
-        { "loop",           OFFSET(loopdev),        proc_string },
-        { "cipher",         OFFSET(cipher),         proc_string },
-        { "ivoffset",       OFFSET(ivoffset),       proc_int64 },
-        { "keyformat",      OFFSET(key.format),     proc_string },
-        { "keyfile",        OFFSET(key.filename),   proc_string },
-        { "keyhash",        OFFSET(key.digestalg),  proc_string },
-        { "keycipher",      OFFSET(key.cipheralg),  proc_string },
-        { "keymaxlen",      OFFSET(key.maxlen),     proc_long },
-        { "passwdretries",  OFFSET(key.retries),    proc_unsigned },
+        { "flags",          OFFSET(flags),
+                            proc_flags,     setup_options },
+        { "bootaction",     OFFSET(flags),
+                            proc_flags,     boot_options },
+        { "dev",            OFFSET(dev),
+                            proc_string,    NULL },
+        { "dir",            OFFSET(dir),
+                            proc_string,    NULL },
+        { "startsector",    OFFSET(start),
+                            proc_int64,     NULL },
+        { "numsectors",     OFFSET(length),
+                            proc_int64,     NULL },
+        { "fstype",         OFFSET(fstype),
+                            proc_string,    NULL },
+        { "mountoptions",   OFFSET(mountoptions),
+                            proc_string,    NULL },
+        { "fsoptions",      OFFSET(mountoptions),
+                            proc_string,    NULL },  /* Deprecated since 4.1 */
+        { "fsckoptions",    OFFSET(fsckoptions),
+                            proc_string,    NULL },
+        { "loop",           OFFSET(loopdev),
+                            proc_string,    NULL },
+        { "cipher",         OFFSET(cipher),
+                            proc_string,    NULL },
+        { "ivoffset",       OFFSET(ivoffset),
+                            proc_int64,     NULL },
+        { "keyformat",      OFFSET(key.format),
+                            proc_string,    NULL },
+        { "keyfile",        OFFSET(key.filename),
+                            proc_string,    NULL },
+        { "keyhash",        OFFSET(key.digestalg),
+                            proc_string,    NULL },
+        { "keycipher",      OFFSET(key.cipheralg),
+                            proc_string,    NULL },
+        { "keymaxlen",      OFFSET(key.maxlen),
+                            proc_long,      NULL },
+        { "passwdretries",  OFFSET(key.retries),
+                            proc_unsigned,  NULL },
         { NULL, 0, NULL }
     };
 #undef OFFSET
@@ -454,7 +506,7 @@ static void read_token(char *buff, unsigned *t_state, tgtdefn_t *tgt)
 
     switch (*t_state) {
         case T_IDENT:
-            (void)proc_string((void*)&tgt->ident, buff);
+            (void)proc_string((void*)&tgt->ident, buff, NULL);
             *t_state = T_LBRACE;
             break;
         case T_LBRACE:
@@ -478,7 +530,8 @@ static void read_token(char *buff, unsigned *t_state, tgtdefn_t *tgt)
             /* Delegate processing to specific token-processor from table: */
             for (tok=toktable; tok->name!=NULL; ++tok) {
                 if (strcmp(tok->name, buff) == 0) {
-                    (*tok->proc)((void*)((char*)tgt + tok->varoffset), eq);
+                    (*tok->proc)((void*)((char*)tgt + tok->varoffset),
+                                    eq, tok->config);
                     break;
                 }
             }
@@ -761,8 +814,8 @@ tgtstat_t *get_tgtstatus(const tgtdefn_t *tgt)
 
 tgtstat_t *get_all_tgtstatus()
     /** Find list of mount/owner status for all mounted targets */
-{   char *fname=NULL;
-    tgtstat_t *ts=NULL, *head=NULL, **sfp=&head;
+{   char *fname = NULL;
+    tgtstat_t *ts = NULL, *head = NULL, **sfp = &head;
     struct statfile *sf;
     int badlock;
 
@@ -844,5 +897,5 @@ int put_tgtstatus(const tgtdefn_t *tgt, const tgtstat_t *newstat)
 }
 
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
