@@ -1,6 +1,6 @@
 /*
  *  declarations for config-table & mount-table utilities for cryptmount
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -29,10 +29,10 @@ struct tgtdefn;
 
 typedef struct tgtstat
 {
-    char *ident;            /* unique identifying name of target */
-    unsigned long uid;      /* user-ID responsible for mounting filesystem */
+    char *ident;            /*!< unique identifying name of target */
+    unsigned long uid;      /*!< user-ID responsible for mounting filesystem */
 
-    struct tgtstat *nx;     /* form into linked list */
+    struct tgtstat *nx;     /*!< form into linked list */
 } tgtstat_t;
 
 
@@ -57,5 +57,5 @@ void free_tgtstatus(tgtstat_t *tstat);
 #endif  /* _TABLES_H */
 
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
