@@ -1,6 +1,6 @@
 /*
  *  Miscellaneous utility functions for cryptmount
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -24,6 +24,7 @@
 #include <config.h>
 
 #include <ctype.h>
+#include <fcntl.h>
 #include <linux/major.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -181,7 +182,7 @@ int ut_test_strings()
 
 
 int cm_path(char **buff, const char *file)
-    /* Create full pathname of config file */
+    /** Create full pathname of config file */
 {   size_t pfxlen, sfxlen;
     const char *pfx=NULL;
 
@@ -204,7 +205,7 @@ int cm_path(char **buff, const char *file)
 
 
 char *cm_strdup(const char *orig)
-    /* Make duplicate of existing string, allocating memory for copy */
+    /** Make duplicate of existing string, allocating memory for copy */
 {   char *cpy;
 
     if (orig == NULL) return NULL;
@@ -216,7 +217,7 @@ char *cm_strdup(const char *orig)
 
 
 int cm_strcasecmp(const char *s1, const char *s2)
-    /* Find legigraphical order of s1 & s2, ignoring case */
+    /** Find legigraphical order of s1 & s2, ignoring case */
 {
     if (s1 == NULL || s2 == NULL) return (s1 != NULL) - (s2 != NULL);
     while (*s1 != '\0' && *s2 != '\0' && tolower(*s1) == tolower(*s2)) {
@@ -228,7 +229,7 @@ int cm_strcasecmp(const char *s1, const char *s2)
 
 
 int cm_startswith(const char **str, const char *prefix)
-    /* Check whether *prefix appears at start of **str */
+    /** Check whether *prefix appears at start of **str */
 {   int valid=1;
 
     if (str == NULL) return 0;
@@ -270,7 +271,7 @@ int ut_test_strops()
 
 
 void *sec_realloc(void *ptr, size_t size)
-    /* Slightly more secure version of realloc() */
+    /** Slightly more secure version of realloc() */
 {   size_t cnt, *memarr;
 
     cnt = (size + 2 * sizeof(size_t) - 1) / sizeof(size_t);
@@ -302,7 +303,7 @@ void *sec_realloc(void *ptr, size_t size)
 
 
 void mem_cleanse(unsigned char *addr, size_t sz)
-    /* Overwrite memory with (weak) pseudo-random numbers */
+    /** Overwrite memory with (weak) pseudo-random numbers */
 {   size_t i;
     static unsigned long salt=0x917c;
 
@@ -316,7 +317,7 @@ void mem_cleanse(unsigned char *addr, size_t sz)
 
 
 void sec_free(void *ptr)
-    /* Slightly more secure version of free() */
+    /** Slightly more secure version of free() */
 {   size_t *memarr, sz;
 
     if (ptr == NULL) return;
@@ -330,68 +331,75 @@ void sec_free(void *ptr)
 }
 
 
-int get_randkey(unsigned char *buff, size_t len)
-    /* Generate random string of bytes */
+/**
+ *  Generate a random sequence of \a len bytes,
+ *  using a cryptographic-quality iterative message-digest
+ *  applied to various entropy sources including /dev/random.
+ */
+int cm_generate_key(unsigned char *buff, size_t len)
 {   struct rnddev {
         const char *name;
-        unsigned devmaj, devmin;
-        FILE *fp; };
+        short devmaj;
+        short devmin; } *rndsrc;
     struct rnddev devs[] = {
-#if !defined(TESTING)
-        { "/dev/hwrng",     MISC_MAJOR, 183,    NULL },
-        { "/dev/random",    MEM_MAJOR, 8,       NULL },
-#endif
-        { "/dev/urandom",   MEM_MAJOR, 9,       NULL },
-        { NULL, 0, 0, NULL } };
-    cm_sha1_ctxt_t *mdcontext=NULL;
-    unsigned char *mdval, *devbuff=NULL;
-    size_t pos, sz, blklen, mdlen;
-    int i, cnt, eflag=ERR_NOERROR;
-    struct stat sbuff;
-    pid_t pid;
+        { "/dev/urandom",   MEM_MAJOR,      9 },
+        { "/dev/random",    MEM_MAJOR,      8 },
+        { "/dev/hwrng",     MISC_MAJOR,   183 },
+        { NULL, 0, 0 } };
+    const size_t POOL_SIZE = 256, NOISE_CHUNK = 16;
+    unsigned char *mdval, *pool, *devbuff = NULL;
+    size_t pos, step, mdlen;
+    const pid_t pid = getpid();
     struct tms tbuff;
     clock_t clk;
-    static unsigned seed=171;
+    static unsigned seed = 1993;
+    int first = 1, total_entropy = 0, eflag = ERR_NOERROR;
 
+    pool = (unsigned char*)sec_realloc(NULL, POOL_SIZE);
 
-    /* Try to find good-quality random-number devices: */
-    for (i=0,cnt=0; devs[i].name!=NULL; ++i) {
-        if (stat(devs[i].name, &sbuff) != 0) continue;
-        if ((unsigned)major(sbuff.st_rdev) != devs[i].devmaj
-          || (unsigned)minor(sbuff.st_rdev) != devs[i].devmin) continue;
-        if ((devs[i].fp = fopen(devs[i].name,"rb")) != NULL) ++cnt;
-    }
-    if (cnt == 0) {
-        fprintf(stderr, _("No random-number devices found\n"));
-        eflag = WRN_LOWENTROPY;
-    }
-    blklen = (len > CM_SHA1_SIZE ? (size_t)CM_SHA1_SIZE : len);
-    devbuff = (unsigned char*)sec_realloc(NULL, (size_t)blklen);
+    devbuff = (unsigned char*)sec_realloc(NULL, NOISE_CHUNK);
+    for (rndsrc=devs; rndsrc->name!=NULL; ++rndsrc) {
+        struct stat sbuff;
+        ssize_t nread;
+        int fd = -1;
 
+        if (stat(rndsrc->name, &sbuff) != 0) continue;
+        if ((unsigned)major(sbuff.st_rdev) != rndsrc->devmaj
+          || (unsigned)minor(sbuff.st_rdev) != rndsrc->devmin) continue;
 
-    pid = getpid();
+        fd = open(rndsrc->name, O_RDONLY | O_NONBLOCK);
+        if (fd < 0) continue;
 
-    /*  Combine multiple sources of entropy
-        (should still be effective if a strict subset aren't viable): */
-    pos = 0;
-    while (pos < len) {
-        mdcontext = cm_sha1_init();
-
-        /* Fold-in entropy from random-number devices: */
-        for (i=0; devs[i].name!=NULL; ++i) {
-            if (devs[i].fp == NULL) continue;
-            sz = fread((void*)devbuff, (size_t)1, (size_t)blklen, devs[i].fp);
-            if (sz > 0) {
-                cm_sha1_block(mdcontext, devbuff, blklen);
+        if (first) {
+            nread = read(fd, pool, POOL_SIZE);
+            if (nread > 0) total_entropy += nread;
+            first = 0;
+        } else {
+            nread = read(fd, devbuff, NOISE_CHUNK);
+            if (nread > 0) {
+                total_entropy += nread;
+                memmove(pool + nread, pool, (POOL_SIZE - nread));
+                memcpy(pool, devbuff, nread);
             }
         }
 
-        /* Fold-in existing bytes from key: */
-        if (pos > 0) {
-            cm_sha1_block(mdcontext, buff, pos);
-        }
+        close(fd);
+    }
+    sec_free(devbuff);
+
+    if (total_entropy < 32) {
+        fprintf(stderr, _("Too few random-number sources found\n"));
+        eflag = WRN_LOWENTROPY;
+    }
+
+    /* Generate key-bytes by recursive hashing of entropy pool: */
+    pos = 0;
+    while (pos < len) {
+        cm_sha1_ctxt_t *mdcontext = cm_sha1_init();
+
+        /* Fold-in various sources of entropy: */
+        cm_sha1_block(mdcontext, pool, POOL_SIZE);
 
-        /* Fold-in some (weak) sources of entropy: */
         cm_sha1_block(mdcontext, (unsigned char*)&pid, sizeof(pid));
         clk = times(&tbuff);
         cm_sha1_block(mdcontext, (unsigned char*)&clk, sizeof(clk));
@@ -400,20 +408,19 @@ int get_randkey(unsigned char *buff, size_t len)
 
         cm_sha1_final(mdcontext, &mdval, &mdlen);
 
-        sz = ((pos + mdlen) > len ? (len - pos) : mdlen);
-        memcpy((void*)(buff + pos),(const void*)mdval, sz);
+        step = ((pos + mdlen) > len ? (len - pos) : mdlen);
+        memcpy((void*)(buff + pos),(const void*)mdval, step);
+        pos += step;
 
-        pos += sz;
-        seed = seed * 106 + 1283;
+        memmove(pool + mdlen, pool, (POOL_SIZE - mdlen));
+        memcpy(pool, mdval, mdlen);
+        seed = seed * 151 + 1279;
 
         cm_sha1_free(mdcontext);
         sec_free(mdval);
     }
 
-    sec_free((void*)devbuff);
-    for (i=0; devs[i].name!=NULL; ++i) {
-        if (devs[i].fp != NULL) fclose(devs[i].fp);
-    }
+    sec_free((void*)pool);
 
     return eflag;
 }
@@ -588,7 +595,7 @@ unsigned char *km_aug_key(const unsigned char *key, unsigned keylen,
     *kptr = chksum;
     idx = (idx + 1) * sizeof(chksum);
     if (idx < *buffsz) {
-        get_randkey((buff + idx), (*buffsz - idx));
+        cm_generate_key((buff + idx), (*buffsz - idx));
     }
 
     return buff;
@@ -983,5 +990,5 @@ int ut_pwfort()
 #endif  /* TESTING */
 
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
