@@ -1,6 +1,6 @@
 /*
  *  Declarations for miscellaneous utilities for cryptmount
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
 
 /*
@@ -66,7 +66,7 @@ void *sec_realloc(void *ptr, size_t size);
 void mem_cleanse(unsigned char *addr, size_t sz);
 void sec_free(void *ptr);
 
-int get_randkey(unsigned char *key, size_t len);
+int cm_generate_key(unsigned char *key, size_t len);
 int km_get_passwd(const char *ident, const km_pw_context_t *pw_ctxt,
                 char **passwd, int isnew, int verify);
 int cm_confirm(const char *msg);
@@ -126,5 +126,5 @@ static inline int cm_fwrite(const void *buff, size_t nbytes, FILE *stream) {
 #endif  /* _UTILS_H */
 
 /*
- *  (C)Copyright 2005-2012, RW Penney
+ *  (C)Copyright 2005-2013, RW Penney
  */
